/*
 * Copyright (C) 2011 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Neil Jagdish Patel <njpatel@gmail.com>
 */

namespace TabWidgets
{
  public class Entry : Gtk.Entry
  {
    public Entry ()
    {

    }
    
    construct
    {
      //get_style_context ().add_class ("menubar");
      set_has_frame (false);
      set_size_request (-1, 24);

      Gdk.RGBA color;
      get_style_context ().get_background_color (Gtk.StateFlags.NORMAL,
                                                 out color);
      color.red = 1.0f;
      color.green = 0.0f;
      color.blue = 0.0f;
      color.alpha = 0.0f;
      override_background_color (Gtk.StateFlags.NORMAL, color);

      set ("placeholder-text", "Search...");
    }
  }

  public class ToggleButton : Gtk.Button
  {
    private Gtk.Label _label;
    private bool _active;

    public string text { get; construct set; }

    public bool active {
      get { return _active; }
      set
      {
        _active = value;
        toggled (active);
        queue_draw ();
      }
    }

    public signal void toggled (bool active);

    public ToggleButton (string text)
    {
      Object (text:text);
    }

    construct
    {
      _label = new Gtk.Label (text);
      add (_label);

      clicked.connect (() =>
      {
        _active = !_active;
        toggled (_active);
        queue_draw ();
      });
    }

    public override bool draw (Cairo.Context cr)
    {
      if (_active)
      {
        cr.set_line_width (1.0);
        cr.rectangle (1, 1, 5, 5);
        cr.fill ();
      }
      propagate_draw (_label, cr);
      return false;
    }
  }

  public class RadioButton : Gtk.RadioButton
  {
    private Gtk.Label _label;

    public string text { get; construct set; }

    public RadioButton (string text)
    {
      Object (text:text);
    }

    construct
    {
      _label = new Gtk.Label (text);
      _label.set_alignment (0.5f, 0.5f);
      add (_label);

      set ("draw-indicator", false);
    }

    public override bool draw (Cairo.Context cr)
    {
      if (active)
      {
        cr.translate (-1, -1);
        Gtk.Allocation a;
        get_allocation (out a);

        Gdk.RGBA c;
        get_style_context ().get_color (Gtk.StateFlags.NORMAL, out c);

        var pat = new Cairo.Pattern.linear (0, a.height -1, a.width, a.height -1);
        pat.add_color_stop_rgba (0.0f, c.red, c.green, c.blue, 0.0f);
        pat.add_color_stop_rgba (0.5f, c.red, c.green, c.blue, 0.5f);
        pat.add_color_stop_rgba (1.0f, c.red, c.green, c.blue, 0.0f);
        cr.set_source (pat);
        cr.rectangle (0, a.height - 1, a.width, 1);
        cr.fill ();
      }
      propagate_draw (_label, cr);
      return false;
    }
  }

  public class Button : Gtk.Button
  {
    private Gtk.Label _label;

    public string text { get; construct set; }

    public Button (string text)
    {
      Object (text:text);
    }

    construct
    {
      _label = new Gtk.Label (text);
      _label.set_alignment (0.5f, 0.5f);
      add (_label);

      set ("draw-indicator", false);
    }

    public override bool draw (Cairo.Context cr)
    {
      propagate_draw (_label, cr);
      return false;
    }
  }
}
