/*
 * Copyright (C) 2011 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Ken VanDine <ken@vandine.org>
 */

public class UsersItem : TabBarItem
{
  private Gtk.Alignment _align;
  private Gtk.HBox box;
  private Gtk.HBox _view;
  private GwibberGtk.UserView _user_view;
  private GwibberGtk.StreamView _stream_view;
  public Gwibber.Streams streams { get; construct; }
  private Gtk.Label _title;
  private ulong active_hander_id;
  private Gtk.Entry _entry;

  public UsersItem (Gwibber.Streams streams)
  {
    Object (stream:"user", icon_name:"stock_person", tooltip:"Users", streams:streams);
  }

  construct
  {
    // View
    _view = new Gtk.HBox(true, 0);
    _view.set_no_show_all(true);
    _view.show();

    _user_view = new GwibberGtk.UserView ();
    _user_view.show_all ();
    
    _stream_view = new GwibberGtk.StreamView ();
    _stream_view.set_no_show_all (true);
    _stream_view.notify["prepared"].connect (() => {
      _stream_view.set_no_show_all (false);
      _stream_view.show_all ();
    });


    _view.pack_start(_user_view, true, true, 0);
    _view.pack_start(_stream_view, true, true, 0);


    // Toolbar
    _align = new Gtk.Alignment (0.5f, 0.5f, 1.0f, 1.0f);

    box = new Gtk.HBox (false, 12);
    _align.add (box);

    _entry = new Gtk.Entry ();

    _entry.key_press_event.connect ((t, e)=>
    {
      if (e.keyval == 65307) // Escape
      {
        _entry.set_text ("");
        return true;
      }
      return false;
    });

    Gdk.Color fg_color;
    Gdk.Color.parse ("black", out fg_color);

    _entry.modify_fg (Gtk.StateType.NORMAL, fg_color);

    _entry.activate.connect (() =>
    {
      var streams = new Gwibber.Streams ();
      string name = _entry.get_text ();
      if (name.has_prefix ("@"))
        name = name.replace("@", "");
      _entry.set_text ("");
      streams.create ("all", name);
    });

    _entry.set_no_show_all (true);
    //_entry.show_all ();
    box.pack_start (_entry, false, false, 0);
    _entry.size_allocate.connect_after (()=> { queue_draw (); });

    _title = new Gtk.Label ("");
    box.pack_start (_title, false, false, 4);

    _user_view.show_transient.connect((id, name) => {
      _user_view.hide ();
      _stream_view.hide ();
      var filtered_model = streams.transient_filter_model (streams.transient_model, id);
      _stream_view.model = filtered_model;
      _stream_view.stream = stream;
      _title.set_markup("<b>" + name + "</b>");
      _title.show ();
      });


    var but = new TabWidgets.Button ("Saved Users");
    but.clicked.connect(() => {
      _stream_view.hide ();
      _user_view.show_all ();
      _title.hide ();
      });
    box.pack_end (but, false, false, 0);

    active_hander_id = notify["active"].connect(() => {
      if (active)
      {
        _stream_view.hide ();
        _user_view.show_all ();
        disconnect (active_hander_id);
      }
    });

    _align.show_all ();
  }

  public override Gtk.Widget? get_toolbar ()
  {
    return _align;
  }
  
  public override Gtk.Widget? get_view ()
  {
    return _view;
  }
}
