/*
 * Copyright (C) 2010 Neil Jagdish Patel
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * version 3.0 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3.0 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * Authored by Neil Jagdish Patel <njpatel@gmail.com>
 */

using Gwibber;

namespace GwibberGtk
{
  public class StreamViewTile : Gtk.EventBox
  {
    private const int our_padding = 6;

    private Gtk.Alignment align;
    private Gtk.HBox   main_box;
    private Gtk.Alignment lalignment;
    private Gtk.Alignment ralignment;
    private Gtk.EventBox  icon_box;
    private Gtk.Image  icon;
    private Gtk.VBox   vbox;
    private Gtk.HBox   likes_hbox;
    private Gtk.Label  likes_count;
    private new Gtk.Label  name;
    //private new Gtk.Label  source;
    private Gtk.Label  time;
    private Gtk.Label  message;
    private Gtk.Label  reply_to;
    private Gtk.Label  retweeted_by;
    public Gwibber.Utils utils { get; construct set; }
    public bool show_fullname { get; construct set; }
    private uint _update_time_area_id = 0;

    public uint uid = 0;

    private string sender = null;
    private string url = null;

    private string time_string = "";

    private GwibberGtk.ActionBox action_box;

    public StreamViewTile (Gwibber.Utils utils, bool show_fullname)
    {
      Object (above_child:false, visible_window:false, utils:utils, show_fullname:show_fullname);
    }

    construct
    {
      draw.connect (on_draw);

      align = new Gtk.Alignment (0.0f, 0.0f, 1.0f, 1.0f);
      align.set_padding (our_padding, our_padding, our_padding, our_padding);
      add (align);

      main_box = new Gtk.HBox (false, 12);
      align.add (main_box);

      lalignment = new Gtk.Alignment (0.5f, 0.0f, 0.0f, 0.0f);
      ralignment = new Gtk.Alignment (0.5f, 0.0f, 0.0f, 0.0f);
      main_box.pack_start (lalignment, false, false, 0);
      main_box.pack_end (ralignment, false, false, 0);

      icon_box = new Gtk.EventBox ();
      icon_box.set_visible_window (false);
      icon = new Gtk.Image.from_icon_name ("stock_person", Gtk.IconSize.DIALOG);
      icon_box.add(icon);
      lalignment.add (icon_box);

      vbox = new Gtk.VBox (false, 0);
      main_box.pack_start (vbox, true, true, 0);
      
      var hbox = new Gtk.HBox (false, 0);
      vbox.pack_start (hbox, false, false, 2);
      
      var box = new Gtk.HBox (false, 0);
      name = new Gtk.Label ("");
      name.set_markup ("<b></b>");
      name.set_single_line_mode (false);
      name.set_line_wrap (true);
      name.set_line_wrap_mode (Pango.WrapMode.WORD_CHAR);
      name.set_alignment (0.0f, 0.5f);
      hbox.pack_start (name, false, false, 0);

      time = new Gtk.Label ("");
      time.set_markup ("");
      time.set_alignment (0.0f, 1.0f);
      hbox.pack_end (time, false, false, 0);

      action_box = new GwibberGtk.ActionBox ();
      action_box.hide ();
      //action_box.set_no_show_all (true);
      hbox.pack_end (action_box, false, false, 0);

      message = new Gtk.Label ("");
      message.set_selectable (false);
      message.set_single_line_mode (false);
      message.set_line_wrap (true);
      message.set_line_wrap_mode (Pango.WrapMode.WORD_CHAR);
      message.set_alignment (0.0f, 0.0f);
      vbox.pack_start (message, false, false, 2);

      message.activate_link.connect((uri) => {
        if (uri.has_prefix("gwibber:/"))
        {
          debug ("URI is %s", uri);
          var url = new Soup.URI(uri);
          if (url.path.has_suffix ("tag"))
          {
            string acct = "";
            string query = "";
            string[] args = url.query.split("&");
            foreach (var x in args)
            {
              var args2 = x.split("=");
              if (args2[0] == "acct")
                acct = args2[1];
              if (args2[0] == "query")
                query = "#" + args2[1];
            }
            if (query.length > 0)
            {
              var searches_service = new Gwibber.Searches ();
              var searches = searches_service.list ();
              var _id = searches_service.create(query, query);
            }
          }
          else if (url.path.has_suffix ("user"))
          {
            string acct = "";
            string name = "";
            string[] args = url.query.split("&");
            foreach (var x in args)
            {
              var args2 = x.split("=");
              if (args2[0] == "acct")
                acct = args2[1];
              if (args2[0] == "name")
                name = args2[1];
            }
            if (name.length > 0 && acct.length > 0)
            {
              var streams_service = new Gwibber.Streams ();
              var streams_list = streams_service.list ();
              var _id = streams_service.create(acct, name);
            }
          }
          return false;
        }
        });

      reply_to = new Gtk.Label ("");
      reply_to.set_markup ("<span font_weight='light' font_size='small'></span>");
      reply_to.set_alignment (0.0f, 0.5f);
      vbox.pack_start (reply_to, false, false, 2);

      retweeted_by = new Gtk.Label ("");
      retweeted_by.set_markup ("<span font_weight='light' font_size='small'></span>");
      retweeted_by.set_alignment (0.0f, 0.5f);
      vbox.pack_start (retweeted_by, false, false, 2);

      likes_hbox = new Gtk.HBox (false, 0);
      likes_hbox.set_no_show_all (true);
      // FIXME: Find a theme icon
      var likes_icon = new Gtk.Image.from_file ("/usr/share/gwibber/ui/icons/streams/16x16/favorite.png");
      likes_hbox.pack_start (likes_icon, false, false, 2);
      likes_count = new Gtk.Label ("");
      likes_hbox.pack_start (likes_count, false, false, 0);
      vbox.pack_start (likes_hbox, false, false, 2);

      enter_notify_event.connect (() => {
        update_time_area_wrapper ();
        return false;
      });

      icon_box.enter_notify_event.connect (() => {
        update_time_area_wrapper ();
        return false;
      });

      message.enter_notify_event.connect (() => {
        update_time_area_wrapper ();
        return false;
      });

      leave_notify_event.connect (() => {
        /*
        if (_update_time_area_id != 0)
        {
          Source.remove(_update_time_area_id);
          _update_time_area_id = 0;
        }
        if (!(time.visible))
          update_time_area ();
        */
        update_time_area_wrapper ();
        return false;
      });

      Notify.init ("Gwibber");
    }


    private bool on_draw (Cairo.Context cr)
    {
      Gtk.Allocation alloc;
      get_allocation (out alloc);

      cr.set_line_width (1.0);

      cr.set_source_rgba (1.0, 1.0, 1.0, 1.0);
      cr.move_to (alloc.x, 0.5);
      cr.line_to (alloc.x + alloc.width, 0.5);
      cr.stroke ();

      var pat = new Cairo.Pattern.linear (0, 0, 0, alloc.height);
      pat.add_color_stop_rgba (0.0f, 1.0f, 1.0f, 1.0f, 0.2f);
      pat.add_color_stop_rgba (1.0f, 1.0f, 1.0f, 1.0f, 0.0f);

      cr.rectangle (0, 0, alloc.width, alloc.height);
      cr.set_source (pat);
      cr.fill ();

      cr.set_source_rgba (0.0, 0.0, 0.0, 0.1);
      cr.move_to (alloc.x, alloc.height - 0.5);
      cr.line_to (alloc.x + alloc.width, alloc.height - 0.5);
      cr.stroke ();

      propagate_draw (align, cr);

      return true;
    }

    public void reset ()
    {
      hide ();
    }

    [Signal (action=true)]
    public virtual signal void reply (string mid, string account, string sender) 
    {
    }

    private void update_time_area_wrapper ()
    {
      if (_update_time_area_id == 0)
      {
        _update_time_area_id = Timeout.add (200, () => {
          update_time_area ();
          return false;
        });
      }
    }

    private void update_time_area ()
    {
      int x, y;
      get_pointer (out x, out y);
      Gtk.Allocation a;
      get_allocation (out a);
      if (x > 0 && x < a.width && y > 0 && y < a.height)
      {
        time.hide ();
        action_box.show_all ();
      }
      else
      {
        var str = "<span font_weight='light' font_size='small' color='#999'>";
        str += time_string;
        str += "</span>";
        time.set_markup (str);
        action_box.hide ();
        time.show_all ();
      }
      _update_time_area_id = 0;
    }

    public void set_details (string[] _accounts,
                             string _stream,
                             string _sender,
                             string _sender_nick,
                             bool _from_me,
                             string _timestamp,
                             string _message,
                             string _html,
                             string _icon_uri,
                             string _url,
                             string _source,
                             string _timestring,
                             string _reply_nick,
                             string _reply_name,
                             string _reply_url,
                             double _likes,
                             string _retweet_nick,
                             string _retweet_name,
                             string _retweet_id,
                             string _link_picture,
                             string _link_name,
                             string _link_url,
                             string _link_desc,
                             string _link_caption,
                             string _link_icon,
                             string _img_url,
                             string _img_src,
                             string _img_thumb,
                             string _img_name)
    {

      message.set_markup ("");

      string display_name = _sender;

      if (show_fullname && _sender.length > 0)
        display_name = _sender;
      else if (_sender_nick.length > 0)
        display_name = _sender_nick;

      name.set_markup ("<b>" + display_name + "</b>");

      sender = _sender_nick.length > 0 ? "@" + _sender_nick : "";
      /* FIXME: we need access to settings for this"
      if (settings_map["append_colon"])
        sender += ":";
      */

      /* iterate over the accounts, display the service icon and attach a menu 
       * for actions.
       * this is a temporary hack until njpatel finishes the proper actions widget
       */
      string[] icon_displayed = null;

      /* FIXME: _accounts is made up of account:service:mid
       * we need to allow actions depending on each account/service/mid
       */

      foreach (var _p in action_box.get_children ())
      {
        GwibberGtk.ActionBoxItem _i = _p as GwibberGtk.ActionBoxItem;
        foreach (var _w in _i.get_children ())
        {
          if (_w is Gtk.Menu?)
          {
            Gtk.Menu _y = _w as Gtk.Menu;
            debug ("_y before has %u children", _y.get_children ().length());
            foreach (var _x in _y.get_children ())
            {
              _y.remove (_x);
              _x.destroy ();
              _x = null;
            }
            debug ("_y after has %u children", _y.get_children ().length());
          }
          _i.remove (_w);
          _w.destroy ();
          _w = null;
        }
        action_box.remove (_i);
        _i.destroy ();
        _i.dispose ();
        _i = null;
      }


      //action_box = null;
      //action_box = new GwibberGtk.ActionBox ();

      foreach (var _a in _accounts)
      {
        string _account;
        string _service;
        string _mid;
        (_account, _service, _mid) = _a.split(":");
        if (_account in icon_displayed)
          continue;
        icon_displayed += _account;
        var action_item = new GwibberGtk.ActionBoxItem (_service, _account, _mid, sender);
        action_box.add(action_item);
        action_item.reply.connect((mid, account, sender) => {
          reply (mid, account, sender);
        });

      }

      if (_from_me)
        icon_box.reparent(ralignment);
      else
        icon_box.reparent(lalignment);

      var _avatar_cache_image = utils.avatar_path(_icon_uri);
      if (_avatar_cache_image != null)
        icon.set_from_file(_avatar_cache_image);
      else
        icon.set_from_icon_name ("stock_person", Gtk.IconSize.DIALOG);
 

      /* We need to make this perform better before enabling it
      if (_cache_avatar_id > 0)
        GLib.Source.remove(_cache_avatar_id);

      var _avatar_cache_image = utils.avatar_path(_icon_uri);
      if (_avatar_cache_image != null)
      {
        icon.set_from_file(_avatar_cache_image);
      } else
      {
        _cache_avatar_id = Timeout.add(100, () => {
          _avatar_cache_image = utils.avatar_path (_icon_uri);
          icon.set_from_file(_avatar_cache_image);
          return false;
          });
      }
      */

      url = _url;
      
      icon_box.button_press_event.disconnect(on_avatar_click);
      icon_box.button_press_event.connect(on_avatar_click);

      if ((int)_likes == 0) {
        likes_hbox.hide();
        likes_hbox.set_no_show_all (true);
      } else if ((int)_likes < 0) {
        likes_count.set_markup("");
        likes_hbox.set_no_show_all (false);
        likes_hbox.show_all();
      } else if ((int)_likes == 1) {
        likes_count.set_markup("<span font_size='small'>" + _likes.to_string() + " " + _("person liked this") + "</span>");
        likes_hbox.set_no_show_all (false);
        likes_hbox.show_all();
      } else if ((int)_likes > 1) {
        likes_count.set_markup("<span font_size='small'>" + _likes.to_string() + " " + _("people liked this") + "</span>");
        likes_hbox.set_no_show_all (false);
        likes_hbox.show_all();
      }

      if (_stream == "links")
      {
        string link_str = "<a href='" + _link_url + "'>" + _link_name + "</a>\n";
        link_str += _link_desc;
        link_str += "\n";
        link_str += _link_caption;
        message.set_markup (link_str);
      } else {
        message.set_markup (_html);
      }

      if (_img_url.length > 0)
      {
        string img_str = _html;
        img_str += "\n\n";
        if (_img_name.length > 0)
          img_str += "<a href='" + _img_url + "'>" + _img_name + "</a>";
        else
          img_str += "<a href='" + _img_url + "'>" + _img_url + "</a>";
        message.set_markup (img_str);
      }
       

      time_string = utils.generate_time_string ((uint)long.parse(_timestamp));

      update_time_area_wrapper ();

      reply_to.hide ();
      string reply_to_string =  "<span font_weight='light' font_size='small' color='#999'>";
      if (show_fullname && (_reply_name != null && _reply_name.length > 0))
      {
        reply_to_string += " " + _("in reply to") + " <a href='" + _reply_url + "'>" + _reply_name + "</a>";
        reply_to.show ();
      }
      else if (_reply_nick != null && _reply_nick.length > 0)
      {
        reply_to_string += " " + _("in reply to") + " <a href='" + _reply_url + "'>" + _reply_nick + "</a>";
        reply_to.show ();
      }
      reply_to_string  += "</span>";
      reply_to.set_markup (reply_to_string);

      retweeted_by.hide ();
      string retweeted_by_string =  "<span font_weight='light' font_size='small' color='#999'>";
      if (show_fullname && (_retweet_name != null && _retweet_name.length > 0))
      {
        retweeted_by_string += " " + _("shared by") + " " + _retweet_name;
        retweeted_by.show ();
      }
      else if (_retweet_nick != null && _retweet_nick.length > 0)
      {
        retweeted_by_string += " " + _("shared by") + " " + _retweet_nick;
        retweeted_by.show ();
      }
      retweeted_by_string  += "</span>";
     
      retweeted_by.set_markup (retweeted_by_string);


      queue_resize ();
      show ();
    }                        


    private bool on_avatar_click (Gdk.EventButton button)
    {
        /* FIXME: Display a user view with user info and posts instead of 
        launching external browser */
        try {
          AppInfo.launch_default_for_uri (url, null);
        } catch {
          warning ("Unable to launch: " + url);
        }
        return false;
    }
  }


}
